require ("prototypes.values")

local eb_enemy_autoplace
local autoplace_vanilla = enemy_autoplace
local autoplace_temperature = require ("prototypes.enemy-autoplace")

if settings.startup["eb-disable-temperature-check"].value then
eb_enemy_autoplace = autoplace_vanilla
else
eb_enemy_autoplace = autoplace_temperature
end





local small_biter_scale = 0.5
local medium_biter_scale = 0.7
local big_biter_scale = 1.0
local behemoth_biter_scale = 1.2
local leviathan_scale=2.5

local ebiter_tint1 = {r=1, g=0.3, b=0.3, a=0.95}
local ebiter_tint2 = {r=0.88, g=0.94, b=0, a=0.9}

local HEALTH_S = settings.startup["eb-HealthScaler"].value
local DAMAGE_S = settings.startup["eb-DamageScaler"].value



data:extend(
{

 {
    type = "unit",
    name = "small-explosive-biter",
    icon = "__base__/graphics/icons/small-biter.png",
    icon_size = 32,
    flags = {"placeable-player", "placeable-enemy", "placeable-off-grid", "not-repairable", "breaths-air"},
    max_health = 15*HEALTH_S,
    order = "b-b-a",
    subgroup="enemies",
    resistances =fire_resists,
    healing_per_tick = 0.01,
    collision_box = {{-0.2, -0.2}, {0.2, 0.2}},
    selection_box = {{-0.4, -0.7}, {0.7, 0.4}},
    attack_parameters =
    {
      type = "projectile",
      range = 0.5,
      cooldown = 35,
      ammo_category = "melee",
      ammo_type = make_unit_melee_attack_type(8*DAMAGE_S),
      sound = make_biter_roars(0.4),
      animation = biterattackanimation(small_biter_scale, ebiter_tint1, ebiter_tint2)
    },
    vision_distance = 30,
    movement_speed = 0.2,
    distance_per_frame = 0.1,
    pollution_to_join_attack = 200,
    distraction_cooldown = 300,
    min_pursue_time = 10 * 60,
    max_pursue_distance = 50,
    corpse = "small-explosive-biter-corpse",
    dying_explosion = "blood-explosion-small",
    dying_sound =  make_biter_dying_sounds(0.4),
    working_sound =  make_biter_calls(0.3),
    run_animation = biterrunanimation(small_biter_scale, ebiter_tint1, ebiter_tint2),
	ai_settings = biter_ai_settings
  },

  
  add_biter_die_animation(small_biter_scale, ebiter_tint1, ebiter_tint2,
  {
    type = "corpse",
    name = "small-explosive-biter-corpse",
    icon = "__base__/graphics/icons/small-biter-corpse.png",
    icon_size = 32,
    selection_box = {{-0.8, -0.8}, {0.8, 0.8}},
    selectable_in_game = false,
    subgroup="corpses",
    order = "c[corpse]-a[biter]-a[small]",
    flags = {"placeable-neutral", "placeable-off-grid", "building-direction-8-way", "not-repairable", "not-on-map"},
	dying_speed = 0.04,
    time_before_removed = 15 * 60 * 60,
  }),
	  
  
  
  {
    type = "unit",
    name = "medium-explosive-biter",
    icon = "__base__/graphics/icons/medium-biter.png",
    icon_size = 32,
    flags = {"placeable-player", "placeable-enemy", "placeable-off-grid", "breaths-air", "not-repairable"},
    max_health = 75*HEALTH_S,
    order="b-b-b",
    subgroup="enemies",
    resistances =fire_resists,
    healing_per_tick = 0.01,
    collision_box = {{-0.3, -0.3}, {0.3, 0.3}},
    selection_box = {{-0.7, -1.5}, {0.7, 0.3}},
    sticker_box = {{-0.3, -0.5}, {0.3, 0.1}},
    distraction_cooldown = 300,
    min_pursue_time = 10 * 60,
    max_pursue_distance = 50,
    attack_parameters =
    {
      type = "projectile",
      ammo_category = "melee",
      ammo_type = make_unit_melee_attack_type(16*DAMAGE_S),
      range = 1,
      cooldown = 35,
      sound = make_biter_roars(0.5),
      animation = biterattackanimation(medium_biter_scale, ebiter_tint1, ebiter_tint2)
    },
    vision_distance = 30,
    movement_speed = 0.185,
    distance_per_frame = 0.15,
    -- in pu
    pollution_to_join_attack = 1000,
    corpse = "medium-explosive-biter-corpse",
    dying_explosion = "blood-explosion-small",
    working_sound = make_biter_calls(0.4),
    dying_sound = make_biter_dying_sounds(0.5),
    run_animation = biterrunanimation(medium_biter_scale, ebiter_tint1, ebiter_tint2),
	ai_settings = biter_ai_settings,
  },

  {
    type = "unit",
    name = "big-explosive-biter",
    order="b-b-c",
    icon = "__base__/graphics/icons/big-biter.png",
    icon_size = 32,
    flags = {"placeable-player", "placeable-enemy", "placeable-off-grid", "breaths-air", "not-repairable"},
    max_health = 375*HEALTH_S,
    subgroup="enemies",
    resistances =fire_resists,
    spawning_time_modifier = 3,
    healing_per_tick = 0.02,
    collision_box = {{-0.4, -0.4}, {0.4, 0.4}},
    selection_box = {{-0.7, -1.5}, {0.7, 0.3}},
    sticker_box = {{-0.6, -0.8}, {0.6, 0}},
    distraction_cooldown = 300,
    min_pursue_time = 10 * 60,
    max_pursue_distance = 50,
    attack_parameters =
    {
      type = "projectile",
      range = 1.5,
      cooldown = 35,
      ammo_category = "melee",
      ammo_type = make_unit_melee_attack_type(30*DAMAGE_S),
      sound =  make_biter_roars(0.6),
      animation = biterattackanimation(big_biter_scale, ebiter_tint1, ebiter_tint2)
    },
    vision_distance = 30,
    movement_speed = 0.17,
    distance_per_frame = 0.2,
    -- in pu
    pollution_to_join_attack = 4000,
    corpse = "big-explosive-biter-corpse",
    dying_explosion = "blood-explosion-big",
    working_sound = make_biter_calls(0.5),
    dying_sound = make_biter_dying_sounds(0.6),
    run_animation = biterrunanimation(big_biter_scale, ebiter_tint1, ebiter_tint2),
	ai_settings = biter_ai_settings,
  },

  {
    type = "unit",
    name = "behemoth-explosive-biter",
    order="b-b-d",
    icon = "__base__/graphics/icons/behemoth-biter.png",
    icon_size = 32,
    flags = {"placeable-player", "placeable-enemy", "placeable-off-grid", "breaths-air", "not-repairable"},
    max_health = 3000*HEALTH_S,
    subgroup="enemies",
    resistances =fire_resists,
    spawning_time_modifier = 12,
    healing_per_tick = 0.1,
    collision_box = {{-0.4, -0.4}, {0.4, 0.4}},
    selection_box = {{-0.7, -1.5}, {0.7, 0.3}},
    sticker_box = {{-0.6, -0.8}, {0.6, 0}},
    distraction_cooldown = 300,
    min_pursue_time = 10 * 60,
    max_pursue_distance = 50,
    attack_parameters =
    {
      type = "projectile",
      range = 1.5,
      cooldown = 50,
      ammo_category = "melee",
      ammo_type = make_unit_melee_attack_type(100*DAMAGE_S),
      sound =  make_biter_roars(0.8),
      animation = biterattackanimation(behemoth_biter_scale, ebiter_tint1, ebiter_tint2)
    },
    vision_distance = 30,
    movement_speed = 0.17,
    distance_per_frame = 0.2,
    -- in pu
    pollution_to_join_attack = 20000,
    corpse = "behemoth-explosive-biter-corpse",
    dying_explosion = "blood-explosion-big",
    working_sound = make_biter_calls(0.7),
    dying_sound = make_biter_dying_sounds(0.8),
    run_animation = biterrunanimation(behemoth_biter_scale, ebiter_tint1, ebiter_tint2),
	ai_settings = biter_ai_settings,
  },
 
  add_biter_die_animation(medium_biter_scale, ebiter_tint1, ebiter_tint2,
  {
    type = "corpse",
    name = "medium-explosive-biter-corpse",
    icon = "__base__/graphics/icons/medium-biter-corpse.png",
    icon_size = 32,
    selection_box = {{-1, -1}, {1, 1}},
    selectable_in_game = false,
    subgroup="corpses",
    order = "c[corpse]-a[biter]-b[medium]",
    flags = {"placeable-neutral", "placeable-off-grid", "building-direction-8-way", "not-on-map"},
	dying_speed = 0.04,
    time_before_removed = 15 * 60 * 60,
  }),  
  
  add_biter_die_animation(big_biter_scale, ebiter_tint1, ebiter_tint2,
  {
    type = "corpse",
    name = "big-explosive-biter-corpse",
    icon = "__base__/graphics/icons/big-biter-corpse.png",
    icon_size = 32,
    selection_box = {{-1, -1}, {1, 1}},
    selectable_in_game = false,
    subgroup="corpses",
    order = "c[corpse]-a[biter]-c[big]",
    flags = {"placeable-neutral", "placeable-off-grid", "building-direction-8-way", "not-on-map"},
	dying_speed = 0.04,
    time_before_removed = 15 * 60 * 60,
  }),  
  
  
  add_biter_die_animation(behemoth_biter_scale, ebiter_tint1, ebiter_tint2,
  {
    type = "corpse",
    name = "behemoth-explosive-biter-corpse",
    icon = "__base__/graphics/icons/big-biter-corpse.png",
    icon_size = 32,
    selection_box = {{-1, -1}, {1, 1}},
    selectable_in_game = false,
    subgroup="corpses",
    order = "c[corpse]-a[biter]-c[big]",
    flags = {"placeable-neutral", "placeable-off-grid", "building-direction-8-way", "not-on-map"},
	dying_speed = 0.04,
    time_before_removed = 15 * 60 * 60,
  }),  
  
   {
    type = "unit",
    name = "explosive-leviathan-biter",
    order="b-b-g",
    icon ="__base__/graphics/icons/big-biter-corpse.png",
	icon_size = 32,
    flags = {"placeable-player", "placeable-enemy", "placeable-off-grid", "breaths-air", "not-repairable"},
    max_health = 80000*HEALTH_S,
    subgroup="enemies",
    resistances = leviathan_resists,
    spawning_time_modifier = 12,
    healing_per_tick = 0.1,
    collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
    selection_box = {{-1.2, -2.4}, {1.2, 0.9}},
    sticker_box = {{-1.2, -2}, {1.2, 0}},
    distraction_cooldown = 300,
    loot =
    {
    },
    min_pursue_time = 10 * 60,
    max_pursue_distance = 50,
    attack_parameters =
    {
      type = "projectile",
      range = 1,
      cooldown = 35,
      ammo_category = "melee",
      ammo_type = 
      {
        category = "melee",
        target_type = "entity",
        action =
        {
          type = "direct",
          action_delivery =
          {
            type = "instant",
            target_effects =
            {
              {
                type = "damage",
                damage = { amount = 50 * DAMAGE_S, type = "physical"}
              },
              {
                type = "damage",
                damage = { amount = 50 * DAMAGE_S, type = "explosion"}
              },
              {
                type = "damage",
                damage = { amount = 50* DAMAGE_S , type = "fire"}
              }
            }
          }
        }
      },
      sound =  make_biter_roars(0.9),
      animation = biterattackanimation(leviathan_scale, ebiter_tint1, ebiter_tint2)
    },
    vision_distance = 50,
    movement_speed = 0.2,
    distance_per_frame = 0.2,
    -- in pu
    pollution_to_join_attack = 10000,
    corpse = "explosive-leviathan-biter-corpse",
    dying_explosion = "blood-explosion-big",
    working_sound = make_biter_calls(1.5),
    dying_sound = make_biter_dying_sounds(1.0),
    run_animation = biterrunanimation(leviathan_scale, ebiter_tint1, ebiter_tint2),
	ai_settings = biter_ai_settings,
  },

  
  add_biter_die_animation(leviathan_scale, ebiter_tint1, ebiter_tint2,
  {
    type = "corpse",
    name = "explosive-leviathan-biter-corpse",
    icon = "__base__/graphics/icons/big-biter-corpse.png",
    icon_size = 32,
    selection_box = {{-1.2, -1.2}, {1.2, 1.2}},
    selectable_in_game = false,
    subgroup="corpses",
    order = "c[corpse]-a[biter]-c[big]",
    flags = {"placeable-neutral", "placeable-off-grid", "building-direction-8-way", "not-on-map"},
	dying_speed = 0.04,
    time_before_removed = 15 * 60 * 60,
  }),  
    
 
 
 
  
  --------------
  --- NEST
 
  {
    type = "unit-spawner",
    name = "explosive-biter-spawner",
    icon = "__base__/graphics/icons/biter-spawner.png",
    icon_size = 32,
    flags = {"placeable-player", "placeable-enemy", "not-repairable"},
    max_health = 800*HEALTH_S,
    order="b-b-g",
    subgroup="enemies",
    resistances = fire_resists,
    working_sound = {
      sound =
      {
        {
          filename = "__base__/sound/creatures/spawner.ogg",
          volume = 1.0
        }
      },
      apparent_volume = 2
    },
    dying_sound =
    {
      {
        filename = "__base__/sound/creatures/spawner-death-1.ogg",
        volume = 1.0
      },
      {
        filename = "__base__/sound/creatures/spawner-death-2.ogg",
        volume = 1.0
      }
    },
    healing_per_tick = 0.02,
    collision_box = {{-3.2, -2.2}, {2.2, 2.2}},
    map_generator_bounding_box = {{-4.2, -3.2}, {3.2, 3.2}},
    selection_box = {{-3.5, -2.5}, {2.5, 2.5}},
    -- in ticks per 1 pu
    pollution_absorption_absolute = 10,
    pollution_absorption_proportional = 0.01,	
    corpse = "explosive-biter-spawner-corpse",
    dying_explosion = "blood-explosion-huge",
    max_count_of_owned_units = 10,
    max_friends_around_to_spawn = 7,
    animations =
    {
      spawner_idle_animation(0, ebiter_tint2),
      spawner_idle_animation(1, ebiter_tint2),
      spawner_idle_animation(2, ebiter_tint2),
      spawner_idle_animation(3, ebiter_tint2)
    },
    result_units = (function()
                     local res = {}
                     res[1] = {"small-explosive-biter", {{0.0, 0.3}, {0.6, 0.0}}}
					 res[2] = {"medium-explosive-biter", {{0.2, 0.0}, {0.6, 0.3}, {0.7, 0.1}}}
                     res[3] = {"small-explosive-spitter", {{0.25, 0.0}, {0.5, 0.3}, {0.7, 0.0}}}
					 res[4] = {"medium-explosive-spitter", {{0.4, 0.0}, {0.7, 0.3}, {0.9, 0.1}}}
                     res[5] = {"big-explosive-biter", {{0.5, 0.0}, {1.0, 0.4}}}
					 res[6] = {"big-explosive-spitter", {{0.5, 0.0}, {1.0, 0.4}}}
                     res[7] = {"behemoth-explosive-biter", {{0.9, 0.0}, {1.0, 0.3}}}
					 res[8] = {"behemoth-explosive-spitter", {{0.9, 0.0}, {1.0, 0.3}}}
					 res[9] = {"explosive-leviathan-biter", {{0.965, 0.0}, {1.0, 0.03}}}
					 res[10]= {"leviathan-explosive-spitter", {{0.965, 0.0}, {1.0, 0.03}}}
					 if not settings.startup["eb-disable-mother"].value then
						res[11]= {"mother-explosive-spitter", {{0.98, 0.0}, {1.0, 0.02}}} end
                     return res
                   end)(),
    -- With zero evolution the spawn rate is 6 seconds, with max evolution it is 2.5 seconds
    spawning_cooldown = {360, 150},
    spawning_radius = 10,
    spawning_spacing = 3,
    max_spawn_shift = 0,
    max_richness_for_spawn_shift = 100,
    autoplace = eb_enemy_autoplace.enemy_spawner_autoplace(0),
    call_for_help_radius = 50
  },

  {
    type = "corpse",
    name = "explosive-biter-spawner-corpse",
    flags = {"placeable-neutral", "placeable-off-grid", "not-on-map"},
    icon = "__base__/graphics/icons/biter-spawner-corpse.png",
    icon_size = 32,
    collision_box = {{-2, -2}, {2, 2}},
    selection_box = {{-2, -2}, {2, 2}},
    selectable_in_game = false,
    dying_speed = 0.04,
    time_before_removed = 15 * 60 * 60,
    subgroup="corpses",
    order = "c[corpse]-b[biter-spawner]",
    final_render_layer = "remnants",
    animation =
    {
      spawner_die_animation(0, ebiter_tint2),
      spawner_die_animation(1, ebiter_tint2),
      spawner_die_animation(2, ebiter_tint2),
      spawner_die_animation(3, ebiter_tint2)
    },
    ground_patch =
    {
      sheet = spawner_integration()
    }	
  },
 
  
 
  
	}
)