function Check_train_config(train, station)
	debugp("train config" .. station.name)
	for _, config in pairs(station.surface.find_entities_filtered{area={{x=station.position.x - 2,y=station.position.y - 2},{x=station.position.x + 2,y=station.position.y + 2}},type = "constant-combinator", name = "train-config"}) do
		local cb = config.get_or_create_control_behavior()
		cb.parameters = nil
		local locomotives = train.locomotives
		local carriages = train.carriages
		local numLoco = #locomotives.front_movers + #locomotives.back_movers
		local numWagons = #carriages - numLoco
		debugp(numLoco .. " : " .. numWagons)
	--	game.write_file("locomotives",serpent.block(locomotives),{comment=false})
	--	game.write_file("carriages",serpent.block(carriages),{comment=false})
		-- local rolling_stock = ""
		-- local ctype = ""
		-- local count = 0
		-- local idx = 1
		-- for j,carriage in pairs(carriages) do

		-- 	if ctype == carriage.type or ctype == "" then
		-- 		count = count + 1
		-- 		ctype = carriage.type
		-- 	else
				cb.set_signal(1, {	signal = {
									type = "item",
									name = "locomotive"
									},
						count = numLoco}
				)
			-- 	ctype = carriage.type
			-- 	count = 1
			-- 	idx = idx + 1
			-- end
			cb.set_signal(2, {	signal = {
									type = "item",
									name = "cargo-wagon"
									},
					count = numWagons}
			)
		--end
		global.train_config = global.train_config or {}
		global.train_config[#global.train_config + 1] = {config = config, station = station}

	end
end

function clear_train_config()
	global.train_config = global.train_config or {}
	if #global.train_config == 0 then return end
	for i,train_config in pairs(global.train_config) do
		if train_config.station.valid == true then
			local trains = train_config.station.get_train_stop_trains()
			local flag = false
			for _, train in pairs(trains) do
				if train.station == train_config.station then
					flag = true
					break
				end
			end
			if flag == false then
				if train_config.config.valid == true then
					local cb = train_config.config.get_or_create_control_behavior()
					cb.parameters = nil
				end
				table.remove(global.train_config,i)
			end
		else
			table.remove(global.train_config,i)
		end
	end
end

function build_schedule(train, backer_name, priority)
	local schedule = {}
	schedule.records = {}
	--debugp("Current scheduled record is " .. train.schedule.current)
	for i = 1,train.schedule.current do
		schedule.records[#schedule.records+1] = train.schedule.records[i]
	end
	local record = {station = backer_name, wait_conditions = {}}
	if priority.wc then
		local compare = "or"
		if priority.wc.rb_and == true then compare = "and" end
		if priority.wc.inc_ef == true then
			if priority.wc.empty == true then 
				record.wait_conditions[#record.wait_conditions+1] = {type = "empty", compare_type = compare}
			elseif priority.wc.full == true then
				record.wait_conditions[#record.wait_conditions+1] = {type = "full", compare_type = compare}
			end
		end
		if priority.wc.inactivity == true then
			local ticks = 60 * priority.wc.inact_int
			record.wait_conditions[#record.wait_conditions+1] = {type = "inactivity", compare_type = compare, ticks = ticks}
		end
		if priority.wc.wait_timer == true then
			local ticks = 60 * priority.wc.wait_int
			record.wait_conditions[#record.wait_conditions+1] = {type = "time", compare_type = compare, ticks = ticks}
		end
	else
		record.wait_conditions[#record.wait_conditions+1] = {type = "empty", compare_type = "or"}
		record.wait_conditions[#record.wait_conditions+1] = {type = "inactivity", compare_type = "or", ticks = 300}
	end
	schedule.records[#schedule.records+1] = record	
	schedule.current = #schedule.records
	-- experimental add in any remaining supply stops
	local n = train.schedule.current + 1
	while train.schedule.records[n] do
		local wait = train.schedule.records[n].wait_conditions
		if wait then
			if wait[1].type == "circuit" then
				schedule.records[#schedule.records+1] = train.schedule.records[n]
			end
		end
		wait = nil
		n = n + 1
	end
	train.manual_mode = true
	train.schedule = schedule
	train.manual_mode = false	
end

function on_gui_checked_state_changed(event)
	local element = event.element
	local player = game.players[event.player_index]
	local gui = mod_gui.get_frame_flow(player)
	local frame = gui.station_frame

	if frame then
		if global.player[player.index].cur_priority == #global.priority+1 then
			save_priority_wc(event)
		else
			update_priority_wc(event)
		end
		

--[[ 		local wc = global.priority[global.player[event.player_index].cur_priority].wc
		if element.name == "rb_or" then
			if wc.rb_or == false then
				global.priority[global.player[event.player_index].cur_priority].wc.rb_or = true
				global.priority[global.player[event.player_index].cur_priority].wc.rb_and = false
			end
		elseif element.name == "rb_and" then
			if wc.rb_and == false then
				global.priority[global.player[event.player_index].cur_priority].wc.rb_or = false
				global.priority[global.player[event.player_index].cur_priority].wc.rb_and = true
			end
		elseif element.name == "efinc" then
			if wc.inc_ef == true then global.priority[global.player[event.player_index].cur_priority].wc.inc_ef = false 
			elseif wc.inc_ef == false then global.priority[global.player[event.player_index].cur_priority].wc.inc_ef = true end
		elseif element.name == "empty" then
			if wc.empty == false then
				global.priority[global.player[event.player_index].cur_priority].wc.empty = true
				global.priority[global.player[event.player_index].cur_priority].wc.full = false
			end
		elseif element.name == "full" then
			if wc.full == false then
				global.priority[global.player[event.player_index].cur_priority].wc.empty = false
				global.priority[global.player[event.player_index].cur_priority].wc.full = true
			end
		elseif element.name == "inactivity" then
			if wc.inactivity == true then global.priority[global.player[event.player_index].cur_priority].wc.inactivity = false 
			elseif wc.inactivity == false then global.priority[global.player[event.player_index].cur_priority].wc.inactivity = true end
		elseif element.name == "wait_timer" then
			if wc.wait_timer == true then global.priority[global.player[event.player_index].cur_priority].wc.wait_timer = false
			elseif wc.wait_timer == false then global.priority[global.player[event.player_index].cur_priority].wc.wait_timer = true end
		end ]]

		gui_open_station_frame(player, mode)
	end
end

function update_priority_wc(event)
	local element = event.element
	local player = game.players[event.player_index]	
	local wc = global.priority[global.player[event.player_index].cur_priority].wc
	if element.name == "rb_or" then
		if wc.rb_or == false then
			global.priority[global.player[event.player_index].cur_priority].wc.rb_or = true
			global.priority[global.player[event.player_index].cur_priority].wc.rb_and = false
		end
	elseif element.name == "rb_and" then
		if wc.rb_and == false then
			global.priority[global.player[event.player_index].cur_priority].wc.rb_or = false
			global.priority[global.player[event.player_index].cur_priority].wc.rb_and = true
		end
	elseif element.name == "efinc" then
		if wc.inc_ef == true then global.priority[global.player[event.player_index].cur_priority].wc.inc_ef = false 
		elseif wc.inc_ef == false then global.priority[global.player[event.player_index].cur_priority].wc.inc_ef = true end
	elseif element.name == "empty" then
		if wc.empty == false then
			global.priority[global.player[event.player_index].cur_priority].wc.empty = true
			global.priority[global.player[event.player_index].cur_priority].wc.full = false
		end
	elseif element.name == "full" then
		if wc.full == false then
			global.priority[global.player[event.player_index].cur_priority].wc.empty = false
			global.priority[global.player[event.player_index].cur_priority].wc.full = true
		end
	elseif element.name == "inactivity" then
		if wc.inactivity == true then global.priority[global.player[event.player_index].cur_priority].wc.inactivity = false 
		elseif wc.inactivity == false then global.priority[global.player[event.player_index].cur_priority].wc.inactivity = true end
	elseif element.name == "wait_timer" then
		if wc.wait_timer == true then global.priority[global.player[event.player_index].cur_priority].wc.wait_timer = false
		elseif wc.wait_timer == false then global.priority[global.player[event.player_index].cur_priority].wc.wait_timer = true end
	end
end

function save_priority_wc(event)
	local element = event.element
	local player = game.players[event.player_index]	
	local wc = global.player[player.index].wc
	if element.name == "rb_or" then
		if wc.rb_or == false then
			global.player[player.index].wc.rb_or = true
			global.player[player.index].wc.rb_and = false
		end
	elseif element.name == "rb_and" then
		if wc.rb_and == false then
			global.player[player.index].wc.rb_or = false
			global.player[player.index].wc.rb_and = true
		end
	elseif element.name == "efinc" then
		if wc.inc_ef == true then global.player[player.index].wc.inc_ef = false 
		elseif wc.inc_ef == false then global.player[player.index].wc.inc_ef = true end
	elseif element.name == "empty" then
		if wc.empty == false then
			global.player[player.index].wc.empty = true
			global.player[player.index].wc.full = false
		end
	elseif element.name == "full" then
		if wc.full == false then
			global.player[player.index].wc.empty = false
			global.player[player.index].wc.full = true
		end
	elseif element.name == "inactivity" then
		if wc.inactivity == true then global.player[player.index].wc.inactivity = false 
		elseif wc.inactivity == false then global.player[player.index].wc.inactivity = true end
	elseif element.name == "wait_timer" then
		if wc.wait_timer == true then global.player[player.index].wc.wait_timer = false
		elseif wc.wait_timer == false then global.player[player.index].wc.wait_timer = true end
	end
end

script.on_event(defines.events.on_gui_value_changed, function(event)

	local player = game.players[event.player_index]
	local value = math.floor(event.element.slider_value)
	local gui = mod_gui.get_frame_flow(player)
	local frame = gui.station_frame	
	if event.element.name == "inact_slider" then
		global.priority[global.player[event.player_index].cur_priority].wc.inact_int = value
		frame.inact_table.inact_int.text = value
	elseif event.element.name == "wait_slider" then
		global.priority[global.player[event.player_index].cur_priority].wc.wait_int = value
		frame.rbtable.wait_int.text = value
	end
end)

script.on_event(defines.events.on_gui_text_changed, function(event)
	local player = game.players[event.player_index]
	local gui = mod_gui.get_frame_flow(player)


	local value = tonumber(string.match(event.element.text, "%d+"))	
	if value == nil then value = 1 end
	if value > 200 then value = 200 end
	if value < 1 then value = 1 end
	if event.element.name == "inact_int" then
		local frame = gui.station_frame	
		global.priority[global.player[event.player_index].cur_priority].wc.inact_int = value
		frame.inact_table.inact_int.text = value
		frame.inact_slider.slider_value = value
	elseif event.element.name == "wait_int" then
		local frame = gui.station_frame	
		global.priority[global.player[event.player_index].cur_priority].wc.wait_int = value
		frame.rbtable.wait_int.text = value
		frame.wait_slider.slider_value = value
	elseif event.element.name == "process_priority" then
		local frame = gui.train_publisher		
		global.publishers[ global.cur_publisher[event.player_index] ].proc_priority = value
		frame.proc_table.process_priority.text = value
	end
end)

function find_best_match(backer_name, train)
	local reqpri = {}
--	local status,err = pcall(function()
	--debugp("look through requests")
	for i, request in  pairs(global.requests) do
		if request == nil then table.remove(global.requests, i) end
		--debugp("look through priorities")
		for j,priority in pairs(global.priority) do
			--debugp(request.priority.name .. " : " .. priority.id.name)
			if (request.priority.name == priority.id.name) and not (request.backer_name == nil) then
				for _, station in ipairs(priority.station) do
					if backer_name == station[2] then
						-- process priority
						if request.proc_priority then
							if request.proc_priority ~= "1" then
								if reqpri.proc_priority then
								--	debugp("found reqpri proc priority")
									if reqpri.proc_priority > request.proc_priority then
										reqpri = {proc_priority = request.proc_priority, request = request, priority = priority, i = i}
									end
								else
									reqpri = {proc_priority = request.proc_priority, request = request, priority = priority, i = i}
								end
							else
								reqpri = {proc_priority = request.proc_priority, request = request, priority = priority, i = i}
								return reqpri
							end
						else
							reqpri = {proc_priority = "1", request = request, priority = priority, i = i}
							return reqpri
						end
					end
				end
			end
		end
	end
	-- local next = next
	-- if next(reqpri) ~= nil then return reqpri end
	-- return nil
	return reqpri
end

function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function pending_req()
	global.pending_req = global.pending_req or {}
	for i,pending in ipairs(global.pending_req) do
		local req = game.surfaces["nauvis"].find_entities_filtered{position=pending,name="train-counter"}[1]
		if req then
			req.operable = false
			req.minable = false
			req.destructible = false
			addPSToTable(req)
			table.remove(global.pending_req,i)
		end
	end
end

function fix_raw_wood()
	for _,priority in pairs(global.priority) do
		if priority.id.name == 'raw-wood' then
			priority.id.name = 'wood'
		end
		if priority.resource.name == 'raw-wood' then
			priority.resource.name = 'wood'
		end
	end
	for _,publisher in pairs(global.publishers) do
		if publisher.priority.name == 'raw-wood' then
			publisher.priority.name = 'wood'
		end
	end
end